/*
 * Copyright (c) 2008-2016 Haulmont.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package com.haulmont.cuba.portal.config;

import com.haulmont.cuba.core.config.Config;
import com.haulmont.cuba.core.config.Property;
import com.haulmont.cuba.core.config.Source;
import com.haulmont.cuba.core.config.SourceType;
import com.haulmont.cuba.core.config.defaults.DefaultBoolean;
import com.haulmont.cuba.core.config.defaults.DefaultString;

/**
 * Portal configuration parameters interface.
 *
 */
@Source(type = SourceType.APP)
public interface PortalConfig extends Config {

    /**
     * Whether to use local invocations instead of HTTPInvoker. Makes sense for improving performance,
     * if the WEB and CORE applications started on the same JVM (same Tomcat instance).
     */
    @Property("cuba.useLocalServiceInvocation")
    @DefaultBoolean(true)
    boolean getUseLocalServiceInvocation();

    @Property("cuba.portal.anonymousUserLogin")
    String getAnonymousUserLogin();

    @Property("cuba.trustedClientPassword")
    String getTrustedClientPassword();

    /**
     * @return Whether to use an login/password authentication on client
     * instead of login/password authentication on middleware.
     */
    @Property("cuba.checkPasswordOnClient")
    @DefaultBoolean(false)
    boolean getCheckPasswordOnClient();

    @Property("cuba.portal.theme")
    @DefaultString("default")
    String getTheme();

    /**
     * Active security profile for a PORTAL client.
     * Security scope specifies which security roles will be loaded for user session.
     * By default it's a UI security scope
     */
    @Property("cuba.portal.securityScope")
    @DefaultString("GENERIC_UI")
    String getSecurityScope();
}