/*
 * Copyright (c) 2008-2019 Haulmont.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package spec.cuba.web.frames

import com.haulmont.cuba.core.app.DataService
import com.haulmont.cuba.core.global.LoadContext
import com.haulmont.cuba.gui.screen.OpenMode
import com.haulmont.cuba.security.app.SecurityScopesService
import com.haulmont.cuba.security.entity.Group
import com.haulmont.cuba.security.entity.User
import com.haulmont.cuba.security.role.RolesService
import com.haulmont.cuba.web.testsupport.proxy.TestServiceProxy
import spec.cuba.web.UiScreenSpec
import spec.cuba.web.frames.screens.TestHostScreen
import spec.cuba.web.frames.screens.TestOpenFrameScreen

@SuppressWarnings(["GroovyAccessibility", "GroovyAssignabilityCheck"])
class LegacyFrameOpenTest extends UiScreenSpec {

    def setup() {
        def group = new Group(name: 'Company')
        def users = [
                new User(login: 'admin', loginLowerCase: 'admin', group: group, active: true, name: 'Administrator'),
                new User(login: 'anonymous', loginLowerCase: 'anonymous', group: group, active: true, name: 'Anonymous')
        ]

        TestServiceProxy.mock(DataService, Mock(DataService) {
            loadList(_) >> { LoadContext lc ->
                if (lc.entityMetaClass == 'sec$User') {
                    return users
                }

                return []
            }
        })
        TestServiceProxy.mock(SecurityScopesService, Mock(SecurityScopesService) {
            getAvailableSecurityScopes() >> {
                return []
            }
        })
        TestServiceProxy.mock(RolesService, Mock(RolesService) {
            getRolesPolicyVersion() >> {
                return 2
            }
        })
    }

    def "open UserBrowser in frame declaratively"() {
        def screens = vaadinUi.screens

        when:
        def mainWindow = screens.create("mainWindow", OpenMode.ROOT)
        screens.show(mainWindow)

        then:
        vaadinUi.topLevelWindow == mainWindow.window

        when:
        def testHostScreen = screens.create('testHostScreen', OpenMode.NEW_TAB)
        testHostScreen.show()

        then:
        testHostScreen instanceof TestHostScreen
        ((TestHostScreen)testHostScreen).userBrowse != null
        ((TestHostScreen)testHostScreen).userBrowse.usersTable != null
    }

    def "open UserBrowser in frame programmatically"() {
        def screens = vaadinUi.screens

        when:
        def mainWindow = screens.create("mainWindow", OpenMode.ROOT)
        screens.show(mainWindow)

        then:
        vaadinUi.topLevelWindow == mainWindow.window

        when:
        def testHostScreen = screens.create('testOpenFrame', OpenMode.NEW_TAB)
        testHostScreen.show()

        then:
        testHostScreen instanceof TestOpenFrameScreen
        ((TestOpenFrameScreen)testHostScreen).userBrowse != null
        ((TestOpenFrameScreen)testHostScreen).userBrowse.usersTable != null
    }
}